@@ -32,6 +32,7 @@
 #include <libgen.h>
 #endif
 #include <stdio.h>
+#include <assert.h>
 #include <fcntl.h>
 #include <sys/stat.h>
 #include <sys/ioctl.h>
@@ -47,21 +48,23 @@
 #include <sys/syscall.h>
 #include <sys/utsname.h>
 #include <termios.h>
+#include <errno.h>
 
 #include "drmtest.h"
-#include "i915_drm.h"
 #include "intel_chipset.h"
-#include "intel_gpu_tools.h"
+#include "intel_io.h"
 #include "igt_debugfs.h"
-#include "../version.h"
+#include "version.h"
 #include "config.h"
 
 #include "igt_core.h"
+#include "igt_aux.h"
 
 /**
  * SECTION:igt_core
  * @short_description: Core i-g-t testing support
  * @title: i-g-t core
+ * @include: igt_core.h
  *
  * This libary implements the core of the i-g-t test support infrastructure.
  * Main features are the subtest enumeration, cmdline option parsing helpers for
@@ -83,6 +86,117 @@
  * To allow this i-g-t provides #igt_fixture code blocks for setup code outside
  * of subtests and automatically skips the subtest code blocks themselves. For
  * special cases igt_only_list_subtests() is also provided.
+ *
+ * # Magic Control Blocks
+ *
+ * i-g-t makes heavy use of C macros which serve as magic control blocks. They
+ * work fairly well and transparently but since C doesn't have full-blown
+ * closures there are caveats:
+ *
+ * - Asynchronous blocks which are used to spawn children internally use fork().
+ *   Which means that nonsensical control flow like jumping out of the control
+ *   block is possible, but it will badly confuse the i-g-t library code. And of
+ *   course all caveats of a real fork() call apply, namely that file
+ *   descriptors are copied, but still point at the original file. This will
+ *   terminally upset the libdrm buffer manager if both parent and child keep on
+ *   using the same open instance of the drm device. Usually everything related
+ *   to interacting with the kernel driver must be reinitialized to avoid such
+ *   issues.
+ *
+ * - Code blocks with magic control flow are implemented with setjmp() and
+ *   longjmp(). This applies to #igt_fixture and #igt_subtest blocks and all the
+ *   three variants to finish test: igt_success(), igt_skip() and igt_fail().
+ *   Mostly this is of no concern, except when such a control block changes
+ *   stack variables defined in the same function as the control block resides.
+ *   Any store/load behaviour after a longjmp() is ill-defined for these
+ *   variables. Avoid such code.
+ *
+ *   Quoting the man page for longjmp():
+ *
+ *   "The values of automatic variables are unspecified after a call to
+ *   longjmp() if they meet all the following criteria:"
+ *    - "they are local to the function that made the corresponding setjmp() call;
+ *    - "their values are changed between the calls to setjmp() and longjmp(); and
+ *    - "they are not declared as volatile."
+ *
+ * # Best Practices for Test Helper Libraries Design
+ *
+ * Kernel tests itself tend to have fairly complex logic already. It is
+ * therefore paramount that helper code, both in libraries and test-private
+ * functions, add as little boilerplate code to the main test logic as possible.
+ * But then dense code is hard to understand without constantly consulting
+ * the documentation and implementation of all the helper functions if it
+ * doesn't follow some clear patterns. Hence follow these established best
+ * practices:
+ *
+ * - Make extensive use of the implicit control flow afforded by igt_skip(),
+ *   igt_fail and igt_success(). When dealing with optional kernel features
+ *   combine igt_skip() with igt_fail() to skip when the kernel support isn't
+ *   available but fail when anything else goes awry. void should be the most
+ *   common return type in all your functions, except object constructors of
+ *   course.
+ *
+ * - The main test logic should have no explicit control flow for failure
+ *   conditions, but instead such assumptions should be written in a declarative
+ *   style.  Use one of the many macros which encapsulate i-g-t's implicit
+ *   control flow.  Pick the most suitable one to have as much debug output as
+ *   possible without polluting the code unecessarily. For example
+ *   igt_assert_cmpint() for comparing integers or do_ioctl() for running ioctls
+ *   and checking their results.  Feel free to add new ones to the libary or
+ *   wrap up a set of checks into a private function to further condense your
+ *   test logic.
+ *
+ * - When adding a new feature test function which uses igt_skip() internally,
+ *   use the &lt;prefix&gt;_require_&lt;feature_name&gt; naming scheme. When you
+ *   instead add a feature test function which returns a boolean, because your
+ *   main test logic must take different actions depending upon the feature's
+ *   availability, then instead use the &lt;prefix&gt;_has_&lt;feature_name&gt;.
+ *
+ * - As already mentioned eschew explicit error handling logic as much as
+ *   possible. If your test absolutely has to handle the error of some function
+ *   the customary naming pattern is to prefix those variants with __. Try to
+ *   restrict explicit error handling to leaf functions. For the main test flow
+ *   simply pass the expected error condition down into your helper code, which
+ *   results in tidy and declarative test logic.
+ *
+ * - Make your library functions as simple to use as possible. Automatically
+ *   register cleanup handlers through igt_install_exit_handler(). Reduce the
+ *   amount of setup boilerplate needed by using implicit singletons and lazy
+ *   structure initialization and similar design patterns.
+ *
+ * - Don't shy away from refactoring common code, even when there are just 2-3
+ *   users and even if it's not a net reduction in code. As long as it helps to
+ *   remove boilerplate and makes the code more declarative the resulting
+ *   clearer test flow is worth it. All i-g-t library code has been organically
+ *   extracted from testcases in this fashion.
+ *
+ * - For general coding style issues please follow the kernel's rules laid out
+ *   in
+ *   [CodingStyle](https://www.kernel.org/doc/Documentation/CodingStyle).
+ *
+ * # Interface with Testrunners
+ *
+ * i-g-t testcase are all executables which should be run as root on an
+ * otherwise completely idle system. The test status is reflected in the
+ * exitcode. #IGT_EXIT_SUCCESS means "success", #IGT_EXIT_SKIP "skip",
+ * #IGT_EXIT_TIMEOUT that some operation "timed out".  All other exit codes
+ * encode a failed test result, including any abnormal termination of the test
+ * (e.g. by SIGKILL).
+ *
+ * On top of that tests may report unexpected results and minor issues to
+ * stderr. If stderr is non-empty the test result should be treated as "warn".
+ *
+ * The test lists are generated at build time. Simple testcases are listed in
+ * tests/single-tests.txt and tests with subtests are listed in
+ * tests/multi-tests.txt. When running tests with subtest from a test runner it
+ * is recommend to run each subtest individually, since otherwise the return
+ * code will only reflect the overall result.
+ *
+ * To do that obtain the lists of subtests with "--list-subtests", which can be
+ * run as non-root and doesn't require the i915 driver to be loaded (or any
+ * intel gpu to be present). Then individual subtests can be run with
+ * "--run-subtest". Usage help for tests with subtests can be obtained with the
+ * "--help" commandline option.
  */
 
 static unsigned int exit_handler_count;
@@ -451,7 +565,7 @@ void igt_skip(const char *f, ...)
 		assert(in_fixture);
 		__igt_fixture_end();
 	} else {
-		exit(77);
+		exit(IGT_EXIT_SKIP);
 	}
 }
 
@@ -517,7 +631,7 @@ void igt_success(void)
  */
 void igt_fail(int exitcode)
 {
-	assert(exitcode != 0 && exitcode != 77);
+	assert(exitcode != IGT_EXIT_SUCCESS && exitcode != IGT_EXIT_SKIP);
 
 	if (!failed_one)
 		igt_exitcode = exitcode;
@@ -528,9 +642,12 @@ void igt_fail(int exitcode)
 	if (test_child)
 		exit(exitcode);
 
-	if (in_subtest)
-		exit_subtest("FAIL");
-	else {
+	if (in_subtest) {
+		if (exitcode == IGT_EXIT_TIMEOUT)
+			exit_subtest("TIMEOUT");
+		else
+			exit_subtest("FAIL");
+	} else {
 		assert(!test_with_subtests || in_fixture);
 
 		if (in_fixture) {
@@ -594,10 +711,10 @@ void igt_exit(void)
 	igt_exit_called = true;
 
 	if (igt_only_list_subtests())
-		exit(0);
+		exit(IGT_EXIT_SUCCESS);
 
 	if (!test_with_subtests)
-		exit(0);
+		exit(IGT_EXIT_SUCCESS);
 
 	/* Calling this without calling one of the above is a failure */
 	assert(skipped_one || succeeded_one || failed_one);
@@ -605,9 +722,9 @@ void igt_exit(void)
 	if (failed_one)
 		exit(igt_exitcode);
 	else if (succeeded_one)
-		exit(0);
+		exit(IGT_EXIT_SUCCESS);
 	else
-		exit(77);
+		exit(IGT_EXIT_SKIP);
 }
 
 /* fork support code */
@@ -923,7 +1040,7 @@ static void fatal_sig_handler(int sig)
  * The handler will be passed the signal number if called due to a signal, or
  * 0 otherwise. Exit handlers can also be used from test children spawned with
  * igt_fork(), but not from within helper processes spawned with
- * igt_helper_process(). The list of exit handlers is reset when forking to
+ * igt_fork_helper(). The list of exit handlers is reset when forking to
  * avoid issues with children cleanup up the parent's state too early.
  */
 void igt_install_exit_handler(igt_exit_handler_t fn)
@@ -1020,7 +1137,7 @@ bool igt_run_in_simulation(void)
 	static int simulation = -1;
 
 	if (simulation == -1)
-		simulation = igt_env_set("INTEL_SIMULATION", false);
+		simulation = igt_check_boolean_env_var("INTEL_SIMULATION", false);
 
 	return simulation;
 }
@@ -1030,13 +1147,23 @@ bool igt_run_in_simulation(void)
  *
  * Skip tests when INTEL_SIMULATION environment variable is set. It uses
  * igt_skip() internally and hence is fully subtest aware.
+ *
+ * Note that in contrast to all other functions which use igt_skip() internally
+ * it is allowed to use this outside of an #igt_fixture block in a test with
+ * subtests. This is because in contrast to most other test requirements,
+ * checking for simulation mode doesn't depend upon the present hardware and it
+ * so makes a lot of sense to have this check in the outermost #igt_main block.
  */
 void igt_skip_on_simulation(void)
 {
 	if (igt_only_list_subtests())
 		return;
 
-	igt_require(!igt_run_in_simulation());
+	if (!in_fixture && !in_subtest) {
+		igt_fixture
+			igt_require(!igt_run_in_simulation());
+	} else
+		igt_require(!igt_run_in_simulation());
 }
 
 /* structured logging */
@@ -1047,7 +1174,7 @@ void igt_skip_on_simulation(void)
  * @format: format string
  * @...: optional arguments used in the format string
  *
- * This is the generic structure logging helper function. i-g-t testcase should
+ * This is the generic structured logging helper function. i-g-t testcase should
  * output all normal message to stdout. Warning level message should be printed
  * to stderr and the test runner should treat this as an intermediate result
  * between SUCESS and FAILURE.
@@ -1074,3 +1201,65 @@ void igt_log(enum igt_log_level level, const char *format, ...)
 		vprintf(format, args);
 	va_end(args);
 }
+
+/**
+ * igt_vlog:
+ * @level: #igt_log_level
+ * @format: format string
+ * @args: variable arguments lists
+ *
+ * This is the generic logging helper function using an explicit varargs
+ * structure and hence useful to implement domain-specific logging
+ * functions.
+ *
+ * If there is no need to wrap up a vararg list in the caller it is simpler to
+ * just use igt_log().
+ */
+void igt_vlog(enum igt_log_level level, const char *format, va_list args)
+{
+	assert(format);
+
+	if (igt_log_level > level)
+		return;
+
+	if (level == IGT_LOG_WARN) {
+		fflush(stdout);
+		vfprintf(stderr, format, args);
+	} else
+		vprintf(format, args);
+}
+
+static void igt_alarm_handler(int signal)
+{
+	/* subsequent tests are skipped */
+	skip_subtests_henceforth = SKIP;
+
+	/* exit with timeout status */
+	igt_fail(IGT_EXIT_TIMEOUT);
+}
+
+/**
+ * igt_set_timeout:
+ * @seconds: number of seconds before timeout
+ *
+ * Stop the current test and skip any subsequent tests after the specified
+ * number of seconds have elapsed. The test will exit with #IGT_EXIT_TIMEOUT
+ * status. Any previous timer is cancelled and no timeout is scheduled if
+ * @seconds is zero.
+ *
+ */
+void igt_set_timeout(unsigned int seconds)
+{
+	struct sigaction sa;
+
+	sa.sa_handler = igt_alarm_handler;
+	sigemptyset(&sa.sa_mask);
+	sa.sa_flags = 0;
+
+	if (seconds == 0)
+		sigaction(SIGALRM, NULL, NULL);
+	else
+		sigaction(SIGALRM, &sa, NULL);
+
+	alarm(seconds);
+}
