@@ -20,444 +20,42 @@
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
  * IN THE SOFTWARE.
  *
+ * Authors:
+ * 	Daniel Vetter <daniel.vetter@ffwll.ch>
+ * 	Damien Lespiau <damien.lespiau@intel.com>
  */
 
 #define _GNU_SOURCE
+#include <unistd.h>
 #include <stdio.h>
 #include <stdarg.h>
 #include <fcntl.h>
 #include <sys/stat.h>
 #include <string.h>
-#include <math.h>
+#include <stdlib.h>
 #include <linux/kd.h>
-#include "drm_fourcc.h"
+#include <errno.h>
+
+#include <i915_drm.h>
 
 #include "drmtest.h"
 #include "igt_kms.h"
-
-/* helpers to create nice-looking framebuffers */
-static int create_bo_for_fb(int fd, int width, int height, int bpp,
-			    bool tiled, uint32_t *gem_handle_ret,
-			    unsigned *size_ret, unsigned *stride_ret)
-{
-	uint32_t gem_handle;
-	int size;
-	unsigned stride;
-
-	if (tiled) {
-		int v;
-
-		/* Round the tiling up to the next power-of-two and the
-		 * region up to the next pot fence size so that this works
-		 * on all generations.
-		 *
-		 * This can still fail if the framebuffer is too large to
-		 * be tiled. But then that failure is expected.
-		 */
-
-		v = width * bpp / 8;
-		for (stride = 512; stride < v; stride *= 2)
-			;
-
-		v = stride * height;
-		for (size = 1024*1024; size < v; size *= 2)
-			;
-	} else {
-		/* Scan-out has a 64 byte alignment restriction */
-		stride = (width * (bpp / 8) + 63) & ~63;
-		size = stride * height;
-	}
-
-	gem_handle = gem_create(fd, size);
-
-	if (tiled)
-		gem_set_tiling(fd, gem_handle, I915_TILING_X, stride);
-
-	*stride_ret = stride;
-	*size_ret = size;
-	*gem_handle_ret = gem_handle;
-
-	return 0;
-}
-
-void kmstest_paint_color(cairo_t *cr, int x, int y, int w, int h,
-			 double r, double g, double b)
-{
-	cairo_rectangle(cr, x, y, w, h);
-	cairo_set_source_rgb(cr, r, g, b);
-	cairo_fill(cr);
-}
-
-void kmstest_paint_color_alpha(cairo_t *cr, int x, int y, int w, int h,
-			       double r, double g, double b, double a)
-{
-	cairo_rectangle(cr, x, y, w, h);
-	cairo_set_source_rgba(cr, r, g, b, a);
-	cairo_fill(cr);
-}
-
-void
-kmstest_paint_color_gradient(cairo_t *cr, int x, int y, int w, int h,
-		     int r, int g, int b)
-{
-	cairo_pattern_t *pat;
-
-	pat = cairo_pattern_create_linear(x, y, x + w, y + h);
-	cairo_pattern_add_color_stop_rgba(pat, 1, 0, 0, 0, 1);
-	cairo_pattern_add_color_stop_rgba(pat, 0, r, g, b, 1);
-
-	cairo_rectangle(cr, x, y, w, h);
-	cairo_set_source(cr, pat);
-	cairo_fill(cr);
-	cairo_pattern_destroy(pat);
-}
-
-static void
-paint_test_patterns(cairo_t *cr, int width, int height)
-{
-	double gr_height, gr_width;
-	int x, y;
-
-	y = height * 0.10;
-	gr_width = width * 0.75;
-	gr_height = height * 0.08;
-	x = (width / 2) - (gr_width / 2);
-
-	kmstest_paint_color_gradient(cr, x, y, gr_width, gr_height, 1, 0, 0);
-
-	y += gr_height;
-	kmstest_paint_color_gradient(cr, x, y, gr_width, gr_height, 0, 1, 0);
-
-	y += gr_height;
-	kmstest_paint_color_gradient(cr, x, y, gr_width, gr_height, 0, 0, 1);
-
-	y += gr_height;
-	kmstest_paint_color_gradient(cr, x, y, gr_width, gr_height, 1, 1, 1);
-}
-
-int kmstest_cairo_printf_line(cairo_t *cr, enum kmstest_text_align align,
-				double yspacing, const char *fmt, ...)
-{
-	double x, y, xofs, yofs;
-	cairo_text_extents_t extents;
-	char *text;
-	va_list ap;
-	int ret;
-
-	va_start(ap, fmt);
-	ret = vasprintf(&text, fmt, ap);
-	assert(ret >= 0);
-	va_end(ap);
-
-	cairo_text_extents(cr, text, &extents);
-
-	xofs = yofs = 0;
-	if (align & align_right)
-		xofs = -extents.width;
-	else if (align & align_hcenter)
-		xofs = -extents.width / 2;
-
-	if (align & align_top)
-		yofs = extents.height;
-	else if (align & align_vcenter)
-		yofs = extents.height / 2;
-
-	cairo_get_current_point(cr, &x, &y);
-	if (xofs || yofs)
-		cairo_rel_move_to(cr, xofs, yofs);
-
-	cairo_text_path(cr, text);
-	cairo_set_source_rgb(cr, 0, 0, 0);
-	cairo_stroke_preserve(cr);
-	cairo_set_source_rgb(cr, 1, 1, 1);
-	cairo_fill(cr);
-
-	cairo_move_to(cr, x, y + extents.height + yspacing);
-
-	free(text);
-
-	return extents.width;
-}
-
-static void
-paint_marker(cairo_t *cr, int x, int y)
-{
-	enum kmstest_text_align align;
-	int xoff, yoff;
-
-	cairo_move_to(cr, x, y - 20);
-	cairo_line_to(cr, x, y + 20);
-	cairo_move_to(cr, x - 20, y);
-	cairo_line_to(cr, x + 20, y);
-	cairo_new_sub_path(cr);
-	cairo_arc(cr, x, y, 10, 0, M_PI * 2);
-	cairo_set_line_width(cr, 4);
-	cairo_set_source_rgb(cr, 0, 0, 0);
-	cairo_stroke_preserve(cr);
-	cairo_set_source_rgb(cr, 1, 1, 1);
-	cairo_set_line_width(cr, 2);
-	cairo_stroke(cr);
-
-	xoff = x ? -20 : 20;
-	align = x ? align_right : align_left;
-
-	yoff = y ? -20 : 20;
-	align |= y ? align_bottom : align_top;
-
-	cairo_move_to(cr, x + xoff, y + yoff);
-	cairo_set_font_size(cr, 18);
-	kmstest_cairo_printf_line(cr, align, 0, "(%d, %d)", x, y);
-}
-
-void kmstest_paint_test_pattern(cairo_t *cr, int width, int height)
-{
-	paint_test_patterns(cr, width, height);
-
-	cairo_set_line_cap(cr, CAIRO_LINE_CAP_SQUARE);
-
-	/* Paint corner markers */
-	paint_marker(cr, 0, 0);
-	paint_marker(cr, width, 0);
-	paint_marker(cr, 0, height);
-	paint_marker(cr, width, height);
-
-	assert(!cairo_status(cr));
-}
-
-void kmstest_paint_image(cairo_t *cr, const char *filename,
-			 int dst_x, int dst_y, int dst_width, int dst_height)
-{
-	cairo_surface_t *image;
-	int img_width, img_height;
-	double scale_x, scale_y;
-
-	image = cairo_image_surface_create_from_png(filename);
-	assert(cairo_surface_status(image) == CAIRO_STATUS_SUCCESS);
-
-	img_width = cairo_image_surface_get_width(image);
-	img_height = cairo_image_surface_get_height(image);
-
-	scale_x = (double)dst_width / img_width;
-	scale_y = (double)dst_height / img_height;
-
-	cairo_save(cr);
-
-	cairo_translate(cr, dst_x, dst_y);
-	cairo_scale(cr, scale_x, scale_y);
-	cairo_set_source_surface(cr, image, 0, 0);
-	cairo_paint(cr);
-
-	cairo_surface_destroy(image);
-
-	cairo_restore(cr);
-}
-
-#define DF(did, cid, _bpp, _depth)	\
-	{ DRM_FORMAT_##did, CAIRO_FORMAT_##cid, # did, _bpp, _depth }
-static struct format_desc_struct {
-	uint32_t drm_id;
-	cairo_format_t cairo_id;
-	const char *name;
-	int bpp;
-	int depth;
-} format_desc[] = {
-	DF(RGB565,	RGB16_565,	16, 16),
-	DF(RGB888,	INVALID,	24, 24),
-	DF(XRGB8888,	RGB24,		32, 24),
-	DF(XRGB2101010,	RGB30,		32, 30),
-	DF(ARGB8888,	ARGB32,		32, 32),
-};
-#undef DF
-
-#define for_each_format(f)	\
-	for (f = format_desc; f - format_desc < ARRAY_SIZE(format_desc); f++)
-
-static uint32_t bpp_depth_to_drm_format(int bpp, int depth)
-{
-	struct format_desc_struct *f;
-
-	for_each_format(f)
-		if (f->bpp == bpp && f->depth == depth)
-			return f->drm_id;
-
-	abort();
-}
-
-/* Return fb_id on success, 0 on error */
-unsigned int kmstest_create_fb(int fd, int width, int height, int bpp,
-			       int depth, bool tiled, struct kmstest_fb *fb)
-{
-	memset(fb, 0, sizeof(*fb));
-
-	if (create_bo_for_fb(fd, width, height, bpp, tiled, &fb->gem_handle,
-			       &fb->size, &fb->stride) < 0)
-		return 0;
-
-	if (drmModeAddFB(fd, width, height, depth, bpp, fb->stride,
-			       fb->gem_handle, &fb->fb_id) < 0) {
-		gem_close(fd, fb->gem_handle);
-
-		return 0;
-	}
-
-	fb->width = width;
-	fb->height = height;
-	fb->tiling = tiled;
-	fb->drm_format = bpp_depth_to_drm_format(bpp, depth);
-
-	return fb->fb_id;
-}
-
-uint32_t drm_format_to_bpp(uint32_t drm_format)
-{
-	struct format_desc_struct *f;
-
-	for_each_format(f)
-		if (f->drm_id == drm_format)
-			return f->bpp;
-
-	abort();
-}
-
-unsigned int kmstest_create_fb2(int fd, int width, int height, uint32_t format,
-			        bool tiled, struct kmstest_fb *fb)
-{
-	uint32_t handles[4];
-	uint32_t pitches[4];
-	uint32_t offsets[4];
-	uint32_t fb_id;
-	int bpp;
-	int ret;
-
-	memset(fb, 0, sizeof(*fb));
-
-	bpp = drm_format_to_bpp(format);
-	ret = create_bo_for_fb(fd, width, height, bpp, tiled, &fb->gem_handle,
-			      &fb->size, &fb->stride);
-	if (ret < 0)
-		return ret;
-
-	memset(handles, 0, sizeof(handles));
-	handles[0] = fb->gem_handle;
-	memset(pitches, 0, sizeof(pitches));
-	pitches[0] = fb->stride;
-	memset(offsets, 0, sizeof(offsets));
-	if (drmModeAddFB2(fd, width, height, format, handles, pitches,
-			  offsets, &fb_id, 0) < 0) {
-		gem_close(fd, fb->gem_handle);
-
-		return 0;
-	}
-
-	fb->width = width;
-	fb->height = height;
-	fb->tiling = tiled;
-	fb->drm_format = format;
-	fb->fb_id = fb_id;
-
-	return fb_id;
-}
-
-unsigned int kmstest_create_color_fb(int fd, int width, int height,
-				     uint32_t format, bool tiled,
-				     double r, double g, double b,
-				     struct kmstest_fb *fb /* out */)
-{
-	unsigned int fb_id;
-	cairo_t *cr;
-
-	fb_id = kmstest_create_fb2(fd, width, height, format, tiled, fb);
-	igt_assert(fb_id);
-
-	cr = kmstest_get_cairo_ctx(fd, fb);
-	kmstest_paint_color(cr, 0, 0, width, height, r, g, b);
-	igt_assert(cairo_status(cr) == 0);
-	cairo_destroy(cr);
-
-	return fb_id;
-}
-
-static cairo_format_t drm_format_to_cairo(uint32_t drm_format)
-{
-	struct format_desc_struct *f;
-
-	for_each_format(f)
-		if (f->drm_id == drm_format)
-			return f->cairo_id;
-
-	abort();
-}
-
-static void __kmstest_destroy_cairo_surface(void *arg)
-{
-	struct kmstest_fb *fb = arg;
-	munmap(cairo_image_surface_get_data(fb->cairo_surface), fb->size);
-}
-
-cairo_surface_t *kmstest_get_cairo_surface(int fd, struct kmstest_fb *fb)
-{
-	if (fb->cairo_surface == NULL) {
-		fb->cairo_surface =
-			cairo_image_surface_create_for_data(gem_mmap(fd, fb->gem_handle, fb->size, PROT_READ | PROT_WRITE),
-							    drm_format_to_cairo(fb->drm_format),
-							    fb->width, fb->height, fb->stride);
-
-		cairo_surface_set_user_data(fb->cairo_surface,
-					    (cairo_user_data_key_t *)kmstest_get_cairo_surface,
-					    fb, __kmstest_destroy_cairo_surface);
-	}
-
-	gem_set_domain(fd, fb->gem_handle,
-		       I915_GEM_DOMAIN_CPU, I915_GEM_DOMAIN_CPU);
-
-	igt_assert(cairo_surface_status(fb->cairo_surface) == CAIRO_STATUS_SUCCESS);
-	return cairo_surface_reference(fb->cairo_surface);
-}
-
-cairo_t *kmstest_get_cairo_ctx(int fd, struct kmstest_fb *fb)
-{
-	cairo_surface_t *surface;
-	cairo_t *cr;
-
-	surface = kmstest_get_cairo_surface(fd, fb);
-	cr = cairo_create(surface);
-	cairo_surface_destroy(surface);
-
-	igt_assert(cairo_status(cr) == CAIRO_STATUS_SUCCESS);
-
-	return cr;
-}
-
-void kmstest_write_fb(int fd, struct kmstest_fb *fb, const char *filename)
-{
-	cairo_surface_t *surface;
-	cairo_status_t status;
-
-	surface = kmstest_get_cairo_surface(fd, fb);
-	status = cairo_surface_write_to_png(surface, filename);
-	cairo_surface_destroy(surface);
-
-	igt_assert(status == CAIRO_STATUS_SUCCESS);
-}
-
-void kmstest_remove_fb(int fd, struct kmstest_fb *fb)
-{
-	cairo_surface_destroy(fb->cairo_surface);
-	do_or_die(drmModeRmFB(fd, fb->fb_id));
-	gem_close(fd, fb->gem_handle);
-}
-
-const char *kmstest_format_str(uint32_t drm_format)
-{
-	struct format_desc_struct *f;
-
-	for_each_format(f)
-		if (f->drm_id == drm_format)
-			return f->name;
-
-	return "invalid";
-}
-
+#include "igt_aux.h"
+
+/**
+ * SECTION:igt_kms
+ * @short_description: Kernel modesetting support library
+ * @title: i-g-t kms
+ * @include: igt_kms.h
+ *
+ * This library provides support to enumerate and set modeset configurations.
+ *
+ * Since this library is very much still a work-in-progress and the interfaces
+ * still in-flux detailed api documentation is currently still missing.
+ *
+ * Note that this library's header pulls in the [i-g-t framebuffer](intel-gpu-tools-i-g-t-framebuffer.html)
+ * library as a dependency.
+ */
 const char *kmstest_pipe_str(int pipe)
 {
 	const char *str[] = { "A", "B", "C" };
@@ -468,25 +66,6 @@ const char *kmstest_pipe_str(int pipe)
 	return str[pipe];
 }
 
-void kmstest_get_all_formats(const uint32_t **formats, int *format_count)
-{
-	static uint32_t *drm_formats;
-
-	if (!drm_formats) {
-		struct format_desc_struct *f;
-		uint32_t *format;
-
-		drm_formats = calloc(ARRAY_SIZE(format_desc),
-				     sizeof(*drm_formats));
-		format = &drm_formats[0];
-		for_each_format(f)
-			*format++ = f->drm_id;
-	}
-
-	*formats = drm_formats;
-	*format_count = ARRAY_SIZE(format_desc);
-}
-
 struct type_name {
 	int type;
 	const char *name;
@@ -601,6 +180,34 @@ int kmstest_get_pipe_from_crtc_id(int fd, int crtc_id)
 	return pfci.pipe;
 }
 
+void kmstest_set_connector_dpms(int fd, drmModeConnector *connector, int mode)
+{
+	int i, dpms = 0;
+	bool found_it = false;
+
+	for (i = 0; i < connector->count_props; i++) {
+		struct drm_mode_get_property prop;
+
+		prop.prop_id = connector->props[i];
+		prop.count_values = 0;
+		prop.count_enum_blobs = 0;
+		if (drmIoctl(fd, DRM_IOCTL_MODE_GETPROPERTY, &prop))
+			continue;
+
+		if (strcmp(prop.name, "DPMS"))
+			continue;
+
+		dpms = prop.prop_id;
+		found_it = true;
+		break;
+	}
+	igt_assert_f(found_it, "DPMS property not found on %d\n",
+		     connector->connector_id);
+
+	igt_assert(drmModeConnectorSetProperty(fd, connector->connector_id,
+					       dpms, mode) == 0);
+}
+
 static signed long set_vt_mode(unsigned long mode)
 {
 	int fd;
@@ -817,23 +424,18 @@ const char *plane_name(enum igt_plane p)
 	} while (0)
 #define LOG(d, fmt, ...)	igt_display_log(d, fmt, ## __VA_ARGS__)
 
-static int  __attribute__((format(printf, 2, 3)))
+static void  __attribute__((format(printf, 2, 3)))
 igt_display_log(igt_display_t *display, const char *fmt, ...)
 {
 	va_list args;
-	int n, i;
-
-	if (!display->verbose)
-		return 0;
+	int i;
 
 	va_start(args, fmt);
-	n = printf("display: ");
+	igt_debug("display: ");
 	for (i = 0; i < display->log_shift; i++)
-		n += printf("%s", LOG_SPACES);
-	n += vprintf(fmt, args);
+		igt_debug("%s", LOG_SPACES);
+	igt_vlog(IGT_LOG_DEBUG, fmt, args);
 	va_end(args);
-
-	return n;
 }
 
 static void igt_display_log_shift(igt_display_t *display, int shift)
@@ -883,23 +485,8 @@ void igt_display_init(igt_display_t *display, int drm_fd)
 {
 	drmModeRes *resources;
 	drmModePlaneRes *plane_resources;
-	bool verbose;
-	char *env;
 	int i;
 
-	/*
-	 * It's valid to set verbose before the init so we can get debugging
-	 * output for the init() itself.
-	 */
-	verbose = display->verbose;
-	memset(display, 0, sizeof(igt_display_t));
-	display->verbose = verbose;
-
-	/* allow a verbose override from an env variable */
-	env = getenv("IGT_DISPLAY_VERBOSE");
-	if (env)
-		display->verbose = atoi(env) != 0;
-
 	LOG_INDENT(display, "init");
 
 	display->drm_fd = drm_fd;
@@ -993,11 +580,6 @@ void igt_display_init(igt_display_t *display, int drm_fd)
 	LOG_UNINDENT(display);
 }
 
-void igt_display_set_verbose(igt_display_t *display, bool verbose)
-{
-	display->verbose = verbose;
-}
-
 int igt_display_get_n_pipes(igt_display_t *display)
 {
 	return display->n_pipes;
@@ -1415,7 +997,7 @@ igt_plane_t *igt_output_get_plane(igt_output_t *output, enum igt_plane plane)
 	return igt_pipe_get_plane(pipe, plane);
 }
 
-void igt_plane_set_fb(igt_plane_t *plane, struct kmstest_fb *fb)
+void igt_plane_set_fb(igt_plane_t *plane, struct igt_fb *fb)
 {
 	igt_pipe_t *pipe = plane->pipe;
 	igt_display_t *display = pipe->display;
@@ -1452,3 +1034,16 @@ void igt_plane_set_position(igt_plane_t *plane, int x, int y)
 
 	plane->position_changed = true;
 }
+
+void igt_wait_for_vblank(int drm_fd, enum pipe pipe)
+{
+	drmVBlank wait_vbl;
+
+	memset(&wait_vbl, 0, sizeof(wait_vbl));
+
+	wait_vbl.request.type = pipe << DRM_VBLANK_HIGH_CRTC_SHIFT |
+				DRM_VBLANK_RELATIVE;
+	wait_vbl.request.sequence = 1;
+
+	igt_assert(drmWaitVBlank(drm_fd, &wait_vbl) == 0);
+}
