@@ -44,12 +44,16 @@
 #include <sys/stat.h>
 #include <sys/time.h>
 #include <sys/wait.h>
-#include "drm.h"
-#include "i915_drm.h"
+
+#include <drm.h>
+
+#include "ioctl_wrappers.h"
 #include "drmtest.h"
 #include "intel_bufmgr.h"
 #include "intel_batchbuffer.h"
-#include "intel_gpu_tools.h"
+#include "intel_io.h"
+#include "intel_chipset.h"
+#include "igt_aux.h"
 
 static void
 prw_set_bo(drm_intel_bo *bo, uint32_t val, int width, int height)
@@ -204,7 +208,7 @@ static void do_overwrite_source(struct access_mode *mode,
 		mode->set_bo(dst[i], i, width, height);
 	}
 	for (i = 0; i < num_buffers; i++)
-		intel_copy_bo(batch, dst[i], src[i], width, height);
+		intel_copy_bo(batch, dst[i], src[i], width*height*4);
 	for (i = num_buffers; i--; )
 		mode->set_bo(src[i], 0xdeadbeef, width, height);
 	for (i = 0; i < num_buffers; i++)
@@ -221,7 +225,7 @@ static void do_early_read(struct access_mode *mode,
 	for (i = num_buffers; i--; )
 		mode->set_bo(src[i], 0xdeadbeef, width, height);
 	for (i = 0; i < num_buffers; i++)
-		intel_copy_bo(batch, dst[i], src[i], width, height);
+		intel_copy_bo(batch, dst[i], src[i], width*height*4);
 	for (i = num_buffers; i--; )
 		mode->cmp_bo(dst[i], 0xdeadbeef, width, height);
 }
@@ -236,9 +240,9 @@ static void do_gpu_read_after_write(struct access_mode *mode,
 	for (i = num_buffers; i--; )
 		mode->set_bo(src[i], 0xabcdabcd, width, height);
 	for (i = 0; i < num_buffers; i++)
-		intel_copy_bo(batch, dst[i], src[i], width, height);
+		intel_copy_bo(batch, dst[i], src[i], width*height*4);
 	for (i = num_buffers; i--; )
-		intel_copy_bo(batch, dummy, dst[i], width, height);
+		intel_copy_bo(batch, dummy, dst[i], width*height*4);
 	for (i = num_buffers; i--; )
 		mode->cmp_bo(dst[i], 0xabcdabcd, width, height);
 }
@@ -386,7 +390,7 @@ igt_main
 		if (num_buffers > max)
 			num_buffers = max;
 		num_buffers /= 2;
-		printf("using 2x%d buffers, each 1MiB\n", num_buffers);
+		igt_info("using 2x%d buffers, each 1MiB\n", num_buffers);
 	}
 
 	for (i = 0; i < ARRAY_SIZE(access_modes); i++)
