@@ -39,13 +39,18 @@
 #include <sys/wait.h>
 #include <limits.h>
 #include <wordexp.h>
+#include <getopt.h>
 #include <signal.h>
+#include <errno.h>
 
-#include "i915_drm.h"
+#include "ioctl_wrappers.h"
+#include "drmtest.h"
+#include "igt_core.h"
+#include "igt_aux.h"
 #include "intel_bufmgr.h"
 #include "intel_batchbuffer.h"
-#include "intel_gpu_tools.h"
-#include "rendercopy.h"
+#include "intel_io.h"
+#include "intel_chipset.h"
 
 static int devid;
 static int card_index = 0;
@@ -68,7 +73,7 @@ struct option_struct {
 static struct option_struct options;
 
 static void init_buffer(drm_intel_bufmgr *bufmgr,
-			struct scratch_buf *buf,
+			struct igt_buf *buf,
 			drm_intel_bo *bo,
 			int width, int height)
 {
@@ -77,7 +82,6 @@ static void init_buffer(drm_intel_bufmgr *bufmgr,
 	buf->size = width * height * 4;
 	igt_assert(buf->bo);
 	buf->tiling = I915_TILING_NONE;
-	buf->data = buf->cpu_mapping = NULL;
 	buf->num_tiles = width * height * 4;
 	buf->stride = width * 4;
 }
@@ -94,7 +98,7 @@ set_bo(drm_intel_bo *bo, uint32_t val, int width, int height)
 		*vaddr++ = val;
 }
 
-static int
+static void
 cmp_bo(drm_intel_bo *bo, uint32_t val, int width, int height)
 {
 	int size = width * height;
@@ -103,14 +107,13 @@ cmp_bo(drm_intel_bo *bo, uint32_t val, int width, int height)
 	drm_intel_gem_bo_start_gtt_access(bo, false);
 	vaddr = bo->virtual;
 	while (size--) {
-		if (*vaddr++ != val) {
-			printf("%d: 0x%x differs from assumed 0x%x\n",
-			       width * height - size, *vaddr-1, val);
-			return -1;
-		}
+		igt_assert_f(*vaddr++ == val,
+			     "%d: 0x%x differs from assumed 0x%x\n"
+			     "seqno_before_test 0x%x, "
+			     " approximated seqno on test fail 0x%x\n",
+			     width * height - size, *vaddr-1, val,
+			     last_seqno, last_seqno + val * 2);
 	}
-
-	return 0;
 }
 
 static drm_intel_bo *
@@ -137,13 +140,13 @@ static void release_bo(drm_intel_bo *bo)
 	drm_intel_bo_unreference(bo);
 }
 
-static void render_copyfunc(struct scratch_buf *src,
-			    struct scratch_buf *dst,
+static void render_copyfunc(struct igt_buf *src,
+			    struct igt_buf *dst,
 			    int width,
 			    int height)
 {
 	const int src_x = 0, src_y = 0, dst_x = 0, dst_y = 0;
-	render_copyfunc_t rendercopy = get_render_copyfunc(devid);
+	igt_render_copyfunc_t rendercopy = igt_get_render_copyfunc(devid);
 	static int warned = 0;
 
 	if (rendercopy) {
@@ -160,7 +163,7 @@ static void render_copyfunc(struct scratch_buf *src,
 		}
 		igt_assert(dst->bo);
 		igt_assert(src->bo);
-		intel_copy_bo(batch_blt, dst->bo, src->bo, width, height);
+		intel_copy_bo(batch_blt, dst->bo, src->bo, width*height*4);
 		intel_batchbuffer_flush(batch_blt);
 	}
 }
@@ -175,7 +178,7 @@ static void exchange_uint(void *array, unsigned i, unsigned j)
 	i_arr[j] = i_tmp;
 }
 
-static int run_sync_test(int num_buffers, bool verify)
+static void run_sync_test(int num_buffers, bool verify)
 {
 	drm_intel_bufmgr *bufmgr;
 	int max;
@@ -183,10 +186,8 @@ static int run_sync_test(int num_buffers, bool verify)
 	int width = 128, height = 128;
 	int fd;
 	int i;
-	int r = -1;
-	int failed = 0;
 	unsigned int *p_dst1, *p_dst2;
-	struct scratch_buf *s_src, *s_dst;
+	struct igt_buf *s_src, *s_dst;
 
 	fd = drm_open_any();
 	igt_assert(fd >= 0);
@@ -222,12 +223,10 @@ static int run_sync_test(int num_buffers, bool verify)
 	igt_assert(s_dst);
 
 	p_dst1 = malloc(num_buffers * sizeof(unsigned int));
-	if (p_dst1 == NULL)
-		return -ENOMEM;
+	igt_assert(p_dst1);
 
 	p_dst2 = malloc(num_buffers * sizeof(unsigned int));
-	if (p_dst2 == NULL)
-		return -ENOMEM;
+	igt_assert(p_dst2);
 
 	for (i = 0; i < num_buffers; i++) {
 		p_dst1[i] = p_dst2[i] = i;
@@ -252,16 +251,10 @@ static int run_sync_test(int num_buffers, bool verify)
 	if (verify) {
 		for (i = 0; i < num_buffers; i++)
 			intel_copy_bo(batch_blt, dst2[p_dst2[i]], dst1[p_dst1[i]],
-				      width, height);
+				      width*height*4);
 
 		for (i = 0; i < num_buffers; i++) {
-			r = cmp_bo(dst2[p_dst2[i]], i, width, height);
-			if (r) {
-				printf("buffer %d differs, seqno_before_test 0x%x, "
-				       " approximated seqno on test fail 0x%x\n",
-				       i, last_seqno, last_seqno + i * 2);
-				failed = -1;
-			}
+			cmp_bo(dst2[p_dst2[i]], i, width, height);
 		}
 	}
 
@@ -286,8 +279,6 @@ static int run_sync_test(int num_buffers, bool verify)
 	gem_quiescent_gpu(fd);
 
 	close(fd);
-
-	return failed;
 }
 
 static int run_cmd(char *s)
@@ -468,32 +459,27 @@ static uint32_t calc_prewrap_val(void)
 	return (random() % pval);
 }
 
-static int run_test(void)
+static void run_test(void)
 {
-	int r;
-
-	if (strnlen(options.cmd, sizeof(options.cmd)) > 0) {
-		r = run_cmd(options.cmd);
-	} else {
-		r = run_sync_test(options.buffers, true);
-	}
-
-	return r;
+	if (strnlen(options.cmd, sizeof(options.cmd)) > 0)
+		igt_assert(run_cmd(options.cmd) == 0);
+	else
+		run_sync_test(options.buffers, true);
 }
 
 static void preset_run_once(void)
 {
 	igt_assert(write_seqno(1) == 0);
-	igt_assert(run_test() == 0);
+	run_test();
 
 	igt_assert(write_seqno(0x7fffffff) == 0);
-	igt_assert(run_test() == 0);
+	run_test();
 
 	igt_assert(write_seqno(0xffffffff) == 0);
-	igt_assert(run_test() == 0);
+	run_test();
 
 	igt_assert(write_seqno(0xfffffff0) == 0);
-	igt_assert(run_test() == 0);
+	run_test();
 }
 
 static void random_run_once(void)
@@ -507,7 +493,7 @@ static void random_run_once(void)
 	} while (val == 0);
 
 	igt_assert(write_seqno(val) == 0);
-	igt_assert(run_test() == 0);
+	run_test();
 }
 
 static void wrap_run_once(void)
@@ -517,7 +503,7 @@ static void wrap_run_once(void)
 	igt_assert(write_seqno(UINT32_MAX - pw_val) == 0);
 
 	while(!read_seqno())
-		igt_assert(run_test() == 0);
+		run_test();
 }
 
 static void background_run_once(void)
